/*
 * Copyright (c) 1998-99 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache 
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must 
 *    not be used to endorse or promote products derived from this software 
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without 
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache 
 *    Project (http://java.apache.org/)."
 *    
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */

package org.apache.jmeter;

import java.io.*;
import java.awt.*;
import java.net.*;
import java.util.*;
import java.awt.event.*;
import org.apache.jmeter.timers.*;
import org.apache.jmeter.visualizers.*;
import org.apache.jmeter.samplers.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;

/**
 * This is Apache JMeter main class that implements both drawing and action 
 * listening.
 *
 * @author  <a href="mailto:stefano@apache.org">Stefano Mazzocchi</a>
 * @version $Revision: 1.5 $ $Date: 1999/02/24 15:31:43 $
 */
public class JMeter extends JFrame implements ActionListener, ChangeListener {

    public static final String VERSION = "Apache JMeter 1.2";
    public static final String PROPERTIES = "org/apache/jmeter/jmeter.properties";
    public static final String IMAGES = "org/apache/jmeter/images/";
    public static final String URL_SEPARATOR = "";

    int samplers;
    int maxSamplers;
    Vector sampler;
    Hashtable alias;
    Visualizer visualizer;
    Properties properties;
    org.apache.jmeter.timers.Timer timer;
    String threadImage;
    String query;
    String cookieFile;
    Vector urls;
    
    JButton start;
    JButton stop;
    JButton clear;
    JButton cookieButton;
    JSlider slider;
    JCheckBox queryCheck;
    JCheckBox encodeCheck;
    JCheckBox aliasCheck;
    JCheckBox cookieCheck;
    JComboBox urlList;
    JComboBox queryList;
    JComboBox cookieList;
    JComboBox timerList;
    JComboBox visualizerList;
    JFileChooser fileChooser;
    JDialog about;
    JPanel timers;
    JPanel visualizers;
    ImageIcon disabled;
    ImageIcon idle;
    
    JMeter(String file) {
        super(VERSION);

        this.init(file);
        
        this.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }  
        });
        
        this.setLocation(100, 100);
        this.draw();
        this.pack();
        this.setVisible(true);
    }

    /**
     * This method is used by the constructor to load the property
     * file that may even reside in the user space, or in the 
     * classpath under org.apache.jmeter.jmeter.properties
     */
    void init(String file) {
        this.properties = JMeterUtils.getProperties(file);
        this.samplers = (Integer.valueOf(properties.getProperty("threads.start"))).intValue();
        this.maxSamplers = (Integer.valueOf(properties.getProperty("threads.max"))).intValue();
        this.sampler = new Vector(samplers);
        this.alias = JMeterUtils.getAlias(properties);
        this.threadImage = properties.getProperty("threads.image");
        this.cookieFile = properties.getProperty("cookies");
    }
    
    /**
     * Draw the whole thing.
     */
    void draw() {

        // Draw toolbar
        start = JMeterUtils.createButton("start", this);
        stop  = JMeterUtils.createButton("stop", this);
        clear = JMeterUtils.createButton("clear", this);
        stop.setEnabled(false);

        JPanel leftButtons = new JPanel();
        leftButtons.setLayout(new FlowLayout(FlowLayout.LEFT));
        leftButtons.add(start);
        leftButtons.add(clear);
        leftButtons.add(stop);
        
        JPanel rightButtons = new JPanel();
        rightButtons.setLayout(new FlowLayout(FlowLayout.RIGHT));
        rightButtons.add(JMeterUtils.createButton("about", this));
        
        JPanel toolBar = new JPanel();        
        toolBar.setBorder(new BevelBorder(BevelBorder.RAISED));
        toolBar.setLayout(new BorderLayout());
        toolBar.add(leftButtons, BorderLayout.WEST);
        toolBar.add(rightButtons, BorderLayout.EAST);

        // Draw URL Panel        
        Border etched = new EtchedBorder();
        JPanel urlPanel = new JPanel();
        urlPanel.setBorder(new TitledBorder(etched, "URL"));
        GridBagLayout g = new GridBagLayout();
        urlPanel.setLayout(g);
        GridBagConstraints c = new GridBagConstraints();
        c.fill = c.BOTH;
        c.insets = new Insets(3, 3, 3, 3);
        
        JLabel urlLabel = new JLabel("URL:", JLabel.RIGHT);
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = 1;
        c.gridheight = 1;
        c.weightx = 0.0;
        c.weighty = 0.0;
        g.setConstraints(urlLabel, c);
        urlPanel.add(urlLabel);
        
        urlList = new JComboBox();
        urlList.setEditable(true);
        c.gridx = 1;
        c.weightx = 1.0;
        g.setConstraints(urlList, c);
        urlPanel.add(urlList);

        aliasCheck = new JCheckBox("Alias");
        aliasCheck.setFocusPainted(false);
        aliasCheck.setForeground(urlLabel.getForeground());
        c.gridx = 2;
        c.weightx = 0.0;
        g.setConstraints(aliasCheck, c);
        urlPanel.add(aliasCheck);
        
        queryCheck = new JCheckBox("Post:");
        queryCheck.setFocusPainted(false);
        queryCheck.setForeground(urlLabel.getForeground());
        queryCheck.setActionCommand("post");
        queryCheck.addActionListener(this);
        c.gridx = 0;
        c.gridy = 1;
        c.weightx = 0.0;
        g.setConstraints(queryCheck, c);
        urlPanel.add(queryCheck);

        queryList = new JComboBox();
        queryList.setEnabled(false);
        queryList.setEditable(true);
        c.gridx = 1;
        c.weightx = 1.0;
        g.setConstraints(queryList, c);
        urlPanel.add(queryList);

        encodeCheck = new JCheckBox("Encoded");
        encodeCheck.setFocusPainted(false);
        encodeCheck.setForeground(urlLabel.getForeground());
        encodeCheck.setEnabled(false);
        c.gridx = 2;
        c.weightx = 0.0;
        g.setConstraints(encodeCheck, c);
        urlPanel.add(encodeCheck);

        // Draw Cookie Panel
        JPanel cookiePanel = new JPanel();
        cookiePanel.setBorder(new TitledBorder(etched, "Cookies"));
        g = new GridBagLayout();
        cookiePanel.setLayout(g);
        c = new GridBagConstraints();
        c.fill = c.BOTH;
        c.insets = new Insets(3, 3, 3, 3);

        cookieCheck = new JCheckBox("Enable");
        cookieCheck.setFocusPainted(false);
        cookieCheck.setForeground(urlLabel.getForeground());
        cookieCheck.setActionCommand("cookie");
        cookieCheck.addActionListener(this);
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = 1;
        c.gridheight = 1;
        c.weightx = 0.0;
        c.weighty = 0.0;
        g.setConstraints(cookieCheck, c);
        cookiePanel.add(cookieCheck);

        JLabel cookieLabel = new JLabel("File:");
        c.gridx = 1;
        g.setConstraints(cookieLabel, c);
        cookiePanel.add(cookieLabel);

        cookieList = new JComboBox();
        cookieList.setEditable(true);
        if (cookieFile != null) cookieList.addItem(cookieFile);
        cookieList.setEnabled(false);
        c.gridx = 2;
        c.weightx = 1.0;
        g.setConstraints(cookieList, c);
        cookiePanel.add(cookieList);

        cookieButton = new JButton("Browse");
        cookieButton.setEnabled(false);
        cookieButton.setActionCommand("browse");
        cookieButton.addActionListener(this);
        c.gridx = 3;
        c.weightx = 0.0;
        g.setConstraints(cookieButton, c);
        cookiePanel.add(cookieButton);

        // Draw Thread Panel
        JPanel threadPanel = new JPanel();
        threadPanel.setBorder(new TitledBorder(etched, "Threads"));
        threadPanel.setLayout(new BorderLayout());
    
        slider = new JSlider(JSlider.HORIZONTAL, 1, maxSamplers, samplers);
        slider.addChangeListener(this);
        slider.setMajorTickSpacing(2);
        slider.setMinorTickSpacing(1);
        slider.setPaintTicks(true);
        slider.setPaintLabels(true);
        threadPanel.add(slider, BorderLayout.NORTH);

        JPanel ledPanel = new JPanel();
        ledPanel.setLayout(new FlowLayout());
        ledPanel.setBorder(new BevelBorder(BevelBorder.LOWERED));
        disabled = JMeterUtils.getImage(this.threadImage + ".disabled.gif");
        idle = JMeterUtils.getImage(this.threadImage + ".idle.gif");
        for (int i = 0; i < maxSamplers; i++) {
            URLSampler s = new URLSampler(JMeterUtils.getImage(this.threadImage + ".enabled.gif"));
            if (i < samplers) s.setDisabledIcon(idle); else s.setDisabledIcon(disabled);
            s.setEnabled(false);
            sampler.addElement(s);
            ledPanel.add((JLabel) s);
        }
        threadPanel.add(ledPanel, BorderLayout.CENTER);
        
        // Draw Timer Panel
        JPanel timerPanel = new JPanel();
        timerPanel.setBorder(new TitledBorder(etched, "Timers"));
        timerPanel.setLayout(new BorderLayout());
        
        Vector t = JMeterUtils.getTimers(this.properties);
        timerList = new JComboBox(t);
        timerList.setEditable(false);
        timerList.setSelectedIndex(0);
        timerList.addActionListener(this);
        timerList.setActionCommand("timer");
        timerPanel.add(timerList, BorderLayout.NORTH);
        
        timers = new JPanel();
        timers.setLayout(new CardLayout());
        for (Enumeration e = t.elements(); e.hasMoreElements();) {
            JComponent timer = (JComponent) e.nextElement();
            timers.add(timer, timer.toString());
        }
        timerPanel.add(timers, BorderLayout.CENTER);
        
        // Draw Visualizer Panel
        JPanel visualizerPanel = new JPanel();
        visualizerPanel.setBorder(new TitledBorder(etched, "Actions"));
        visualizerPanel.setLayout(new BorderLayout());
        
        Vector a = JMeterUtils.getVisualizers(this.properties);
        visualizerList = new JComboBox(a);
        visualizerList.setEditable(false);
        visualizerList.setSelectedIndex(0);
        visualizerList.addActionListener(this);
        visualizerList.setActionCommand("visualizer");
        visualizerPanel.add(visualizerList, BorderLayout.NORTH);
        
        visualizers = new JPanel();
        visualizers.setLayout(new CardLayout());
        for (Enumeration e = a.elements(); e.hasMoreElements();) {
            JComponent visualizer = (JComponent) e.nextElement();
            visualizers.add(visualizer, visualizer.toString());
        }
        visualizerPanel.add(visualizers, BorderLayout.CENTER);

        // Add all panels to main layout    
        JPanel mainPanel = new JPanel();
        g = new GridBagLayout();
        c = new GridBagConstraints();
        c.fill = c.BOTH;
        mainPanel.setLayout(g);
        mainPanel.setBorder(new EmptyBorder(0, 5, 5, 5));
        c.gridx = 0;
        c.gridy = 0;
        c.gridwidth = 2;
        c.gridheight = 1;
        c.weightx = 1.0;
        c.weighty = 0.0;
        g.setConstraints(urlPanel, c);
        mainPanel.add(urlPanel);
        c.gridy = 1;
        g.setConstraints(cookiePanel, c);
        mainPanel.add(cookiePanel);
        c.gridy = 2;
        c.gridwidth = 1;
        c.weightx = 0.65;
        g.setConstraints(threadPanel, c);
        mainPanel.add(threadPanel);
        c.gridx = 1;
        c.gridy = 2;
        c.gridwidth = 1;
        c.weightx = 0.35;
        g.setConstraints(timerPanel, c);
        mainPanel.add(timerPanel);
        c.gridx = 0;
        c.gridy = 3;
        c.gridwidth = 2;
        c.weighty = 1.0;
        g.setConstraints(visualizerPanel, c);
        mainPanel.add(visualizerPanel);
        
        // Set main panels
        this.getContentPane().add(toolBar, BorderLayout.NORTH);
        this.getContentPane().add(mainPanel, BorderLayout.CENTER);
        
        // Create the file chooser
        this.fileChooser = new JFileChooser();
    }

    /**
     * Action listener that handles all GUI events in this class.
     */
    public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand().equals("start")) {
            start();
        } else if (e.getActionCommand().equals("stop")) {
            stop();
        } else if (e.getActionCommand().equals("clear")) {
            this.visualizer.clear();
        } else if (e.getActionCommand().equals("timer")) {
            this.timer = (org.apache.jmeter.timers.Timer) timerList.getSelectedItem();
            ((CardLayout) timers.getLayout()).show(timers,(String) this.timer.toString());            
        } else if (e.getActionCommand().equals("visualizer")) {
            this.visualizer = (Visualizer) visualizerList.getSelectedItem();
            ((CardLayout) visualizers.getLayout()).show(visualizers,(String) this.visualizer.toString());
            for (int i = 0; i < samplers; i++) {
                ((URLSampler) sampler.elementAt(i)).setVisualizer(this.visualizer);
            }    
        } else if (e.getActionCommand().equals("post")) {
            if (queryCheck.isSelected()) {
                queryList.setEnabled(true);
                queryList.setBackground(this.getBackground());
                encodeCheck.setEnabled(true);
            } else {
                queryList.setEnabled(false);
                queryList.setBackground(urlList.getBackground());
                encodeCheck.setEnabled(false);
            }
        } else if (e.getActionCommand().equals("cookie")) {
            if (cookieCheck.isSelected()) {
                cookieList.setEnabled(true);
                cookieList.setBackground(this.getBackground());
                cookieButton.setEnabled(true);
            } else {
                cookieList.setEnabled(false);
                cookieList.setBackground(urlList.getBackground());
                cookieButton.setEnabled(false);
            }
        } else if (e.getActionCommand().equals("browse")) {
            int retval = fileChooser.showOpenDialog(this);
            if (retval == JFileChooser.APPROVE_OPTION) {
                String file = fileChooser.getSelectedFile().toString();
                cookieList.removeItem(file);
                cookieList.addItem(file);
                cookieList.getEditor().setItem(file);
                cookieList.setSelectedItem(file);
                this.cookieFile = file;
            }
        } else if (e.getActionCommand().equals("about")) {
            about();
        } else {
            // do nothing!
        }
    }
    
    /**
     * Change listener that handles thread slider events.
     */
    public void stateChanged(ChangeEvent e) {
        if (!this.slider.getValueIsAdjusting()) {
            int i = 0;
            
            this.samplers = this.slider.getValue();
            
            while (i < samplers) {
                URLSampler s = (URLSampler) sampler.elementAt(i++);
                s.setDisabledIcon(idle);
            }
            
            while (i < maxSamplers) {
                URLSampler s = (URLSampler) sampler.elementAt(i++);
                s.setDisabledIcon(disabled);
            }
            
            repaint();
        }
    }
    
    /**
     * Get the cookie file
     */
    String getCookies() {
        if (cookieCheck.isSelected()) {
            String cookie = (String) cookieList.getSelectedItem();
            cookieList.removeItem(cookie);
            cookieList.addItem(cookie);
            return cookie;
        } else {
            return null;
        }
    }
        
    /**
     * Get the query string
     */
    String getQuery() {
        if (queryCheck.isSelected()) {
            String query = (String) queryList.getSelectedItem();
            queryList.removeItem(query);
            queryList.addItem(query);
            if (encodeCheck.isSelected()) {
                return URLEncoder.encode(query);
            } else {
                return query;
            }
        } else {
            return null;
        }
    }

    /**
     * Get the vector of valid URL objects
     */
    Vector getURLs() {
        String url = (String) urlList.getSelectedItem();
        urlList.removeItem(url);
        urlList.addItem(url);
        url = (aliasCheck.isSelected()) ? (String) alias.get(url) : url;
        return URLSampler.tryURLs(JMeterUtils.tokenize(url, URL_SEPARATOR), query, this.getRootPane());
    }

    /**
     * Called by start button. Starts up sampling threads.
     */
    void start() {
            
        stop.setEnabled(true);
        start.setEnabled(false);
        slider.setEnabled(false);
        urlList.setEnabled(false);
        queryList.setEnabled(false);
        timerList.setEnabled(false);
        queryCheck.setEnabled(false);
        encodeCheck.setEnabled(false);
        aliasCheck.setEnabled(false);
        cookieCheck.setEnabled(false);
        cookieList.setEnabled(false);
        cookieButton.setEnabled(false);
        
        try {
            this.samplers = slider.getValue();
            this.timer = (org.apache.jmeter.timers.Timer) timerList.getSelectedItem();
            this.visualizer = (Visualizer) visualizerList.getSelectedItem();
            this.query = this.getQuery();
            this.urls = this.getURLs();
            this.cookieFile = this.getCookies();

            if (urls.size() > 0) {
                for (int i = 0; i < samplers; i++) {
                    URLSampler s = (URLSampler) sampler.elementAt(i);
                    s.init(timer, urls, query, cookieFile);
                    s.setVisualizer(visualizer);
                    new Thread(s).start();
                }
            } else {
                stop();
            }
        } catch (Exception e) {
            JMeter.error(e, this);
            stop();
        }
    }
    
    /**
     * Called by stop button. Stops sampling threads.
     */
    void stop() {
        
        stop.setEnabled(false);
        start.setEnabled(true);
        slider.setEnabled(true);
        urlList.setEnabled(true);
        queryList.setEnabled(true);
        timerList.setEnabled(true);
        queryCheck.setEnabled(true);
        encodeCheck.setEnabled(true);
        aliasCheck.setEnabled(true);
        cookieCheck.setEnabled(true);
        cookieList.setEnabled(true);
        cookieButton.setEnabled(true);
        
        // NOTE: I do this instead of calling Thread.stop() because
        // that method is deprecated and there is no valid alternative
        // Calling the die() method does not stop the thread but
        // blocks the thread when it's done its current sampling
        // process and avoids last sample to get collected.
        for (int i = 0; i < samplers; i++) {
            ((URLSampler) sampler.elementAt(i)).die();
        }    
    }

    /**
     * Called by about button. Raises about dialog.
     */
    void about() {
        if (about == null) {
            about = new JDialog(this, "About Apache JMeter...", true);
            
            JLabel jmeter = new JLabel(JMeterUtils.getImage("jmeter.jpg"));
            
            JLabel copyright = new JLabel("Copyright (c) 1998-99 The Java Apache Project", JLabel.CENTER);
            JLabel rights = new JLabel("All Rights Reserved.", JLabel.CENTER);
            JPanel infos = new JPanel();
            infos.setOpaque(false);
            infos.setLayout(new GridLayout(0, 1));
            infos.setBorder(new EmptyBorder(5, 5, 5, 5));
            infos.add(copyright);
            infos.add(rights);
            
            Container panel = about.getContentPane();
            panel.setLayout(new BorderLayout());
            panel.setBackground(Color.white);
            panel.add(jmeter, BorderLayout.NORTH);
            panel.add(infos, BorderLayout.SOUTH);
        }
        
        // NOTE: these lines center the about dialog in the
        // current window. Some older Swing versions have
        // a bug in getLocationOnScreen() and they may not
        // make this behave properly.
        Point p = this.getLocationOnScreen();
        Dimension d1 = this.getSize();
        Dimension d2 = about.getSize();
        about.setLocation(p.x + (d1.width - d2.width) / 2, p.y + (d1.height - d2.height) / 2);
        about.pack();
        about.setVisible(true);
    }

    /**
     * Main method that chooses L&F and created the JMeter object.
     */
    public static void main(String arguments[]) {

        try {
            UIManager.setLookAndFeel(
               UIManager.getCrossPlatformLookAndFeelClassName());
        } catch (Exception e) {
            System.err.println("Couldn't use the Java look and feel: " + e);
        }

        if (arguments.length > 1) {
            new JMeter(arguments[0]);
        } else {
            new JMeter(null);
        }
    }
    
    /**
     * Method used to signal an Exception to the user.
     */
    public static void error(Exception e, Component thrower) {
        JOptionPane.showMessageDialog(thrower, e, "Error", JOptionPane.ERROR_MESSAGE);
    }
    
    /**
     * Method used to fail the application if a fatal error is found.
     */
    public static void fail(String message) {
        System.err.println(message);
        System.exit(1);
    }
}