/*
 * Copyright (c) 1998-99 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must
 *    not be used to endorse or promote products derived from this software
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache
 *    Project (http://java.apache.org/)."
 *
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */

package org.apache.jmeter;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * This class contains the static utility methods used by JMeter.
 *
 * @author  <a href="mailto://stefano@apache.org">Stefano Mazzocchi</a>
 * @version $Revision: 1.1 $ $Date: 1999/02/24 15:31:43 $
 */
class JMeterUtils {

    /**
     * This method is used by the init method to load the property
     * file that may even reside in the user space, or in the
     * classpath under org.apache.jmeter.jmeter.properties
     */
    protected static Properties getProperties(String file) {

        Properties p = new Properties();

        try {
            File f = new File(System.getProperty("user.dir") + File.separator + file);
            p.load(new FileInputStream(f));
        } catch (Exception e) {
            p.clear();
            try {
                p.load(JMeterUtils.class.getClassLoader().getSystemResourceAsStream(JMeter.PROPERTIES));
            } catch (IOException ex) {
                JMeter.fail("Could not read internal resource. Archive is broken.");
            }
        }

        return p;
    }

    /**
     * This looks for the requested image in the classpath
     * under org.apache.jmeter.images.<name>
     */
    protected static ImageIcon getImage(String name) {
        return new ImageIcon(JMeterUtils.class.getClassLoader().getSystemResource(JMeter.IMAGES + name));
    }          

    /**
     * Creates the vector of Timers plugins.
     */
    protected static Vector getTimers(Properties properties) {
        return instantiate(getVector(properties, "timer."), "org.apache.jmeter.timers.Timer");
    }

    /**
     * Creates the vector of visualizer plugins.
     */
    protected static Vector getVisualizers(Properties properties) {
        return instantiate(getVector(properties, "visualizer."), "org.apache.jmeter.visualizers.Visualizer");
    }

    /**
     * Creates the vector of alias strings.
     */
    protected static Hashtable getAlias(Properties properties) {
        return getHashtable(properties, "alias.");
    }

    /**
     * Creates a vector of strings for all the properties that start with a
     * common prefix.
     */
    protected static Vector getVector(Properties properties, String name) {
        Vector v = new Vector();
        Enumeration names = properties.keys();
        while (names.hasMoreElements()) {
            String prop = (String) names.nextElement();
            if (prop.startsWith(name)) {
                v.addElement(properties.getProperty(prop));
            }
        }
        return v;
    }

    /**
     * Creates a vector of strings for all the properties that start with a
     * common prefix.
     */
    protected static Hashtable getHashtable(Properties properties, String name) {
        Hashtable t = new Hashtable();
        Enumeration names = properties.keys();
        while (names.hasMoreElements()) {
            String prop = (String) names.nextElement();
            if (prop.startsWith(name)) {
                t.put(prop.substring(name.length()), properties.getProperty(prop));
            }
        }
        return t;
    }

    /**
     * Instantiate a vector of classes
     */
    protected static Vector instantiate(Vector v, String className) {
        Vector i = new Vector();

        try {
            Class c = Class.forName(className);
                    Enumeration elements = v.elements();
            while (elements.hasMoreElements()) {
                String name = (String) elements.nextElement();
                try {
                    Object o = Class.forName(name).newInstance();
                    if (c.isInstance(o)) {
                        i.addElement(o);
                    }
                } catch (ClassNotFoundException e) {
                    JMeter.fail("Error loading class " + name + ": class is not found");
                } catch (IllegalAccessException e) {
                    JMeter.fail("Error loading class " + name + ": does not have access");
                } catch (InstantiationException e) {
                    JMeter.fail("Error loading class " + name + ": could not instantiate");
                }
            }
        } catch (ClassNotFoundException e) {
            JMeter.fail("Error loading class " + className + ": class is not found");
        }

        return i;
    }

    protected static Vector tokenize(String string, String separator) {
        Vector v = new Vector();
        StringTokenizer s = new StringTokenizer(string, separator);
        while (s.hasMoreTokens()) {
            v.addElement(s.nextToken());
        }
        return v;
    }

    protected static JButton createButton(String name, ActionListener listener) {
        JButton button = new JButton(getImage(name + ".on.gif"));
        button.setDisabledIcon(getImage(name + ".off.gif"));
        button.setRolloverIcon(getImage(name + ".over.gif"));
        button.setPressedIcon(getImage(name + ".down.gif"));
        button.setActionCommand(name);
        button.addActionListener(listener);
        button.setRolloverEnabled(true);
        button.setFocusPainted(false);
        button.setBorderPainted(false);
        button.setOpaque(false);
        button.setPreferredSize(new Dimension(40, 40));
        return button;
    }
}

