/*
 * Copyright (c) 1998-99 The Java Apache Project.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgment:
 *    "This product includes software developed by the Java Apache 
 *    Project (http://java.apache.org/)."
 *
 * 4. The names "Apache JMeter" and "Java Apache Project" must 
 *    not be used to endorse or promote products derived from this software 
 *    without prior written permission.
 *
 * 5. Products derived from this software may not be called "Apache JMeter"
 *    nor may "Java Apache Project" appear in their names without 
 *    prior written permission of the Java Apache Project.
 *
 * 6. Redistributions of any form whatsoever must retain the following
 *    acknowledgment:
 *    "This product includes software developed by the Java Apache 
 *    Project (http://java.apache.org/)."
 *    
 * THIS SOFTWARE IS PROVIDED BY THE JAVA APACHE PROJECT "AS IS" AND ANY
 * EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE JAVA APACHE PROJECT OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Java Apache Project. For more information
 * on the Java Apache Project please see <http://java.apache.org/>.
 *
 */

package org.apache.jmeter.samplers;

import java.io.*;
import java.net.*;
import java.util.*;
import org.apache.jmeter.timers.*;
import org.apache.jmeter.visualizers.*;
import javax.swing.*;

/**
 * This class implements the runnable inteface to provide
 * the action that sampling threads should perform to 
 * sample URL resources.
 *
 * It also extends JLabel to visually inform about its status.
 *
 * @author  <a href="mailto:stefano@apache.org">Stefano Mazzocchi</a>
 * @version $Revision: 1.4 $ $Date: 1999/02/05 16:13:12 $
 */
public class URLSampler extends JLabel implements Runnable {

    private int index;
    private Vector urls;
    private String query;
    private boolean alive;
    private org.apache.jmeter.timers.Timer timer;
    private Visualizer visualizer;
    private CookieManager cookieManager = null; 
    
    public URLSampler(ImageIcon icon) {
        super(icon);
    }
    
    /** 
     * Set the visualizer that currently collects samples
     */
    public void setVisualizer(Visualizer visualizer) {
        this.visualizer = visualizer;
    }

    /** 
     * Initialize the sampler.
     */
    public void init(org.apache.jmeter.timers.Timer timer, Vector urls, String query, String cookies) throws IOException {
        if (cookies != null) this.cookieManager = new CookieManager(cookies);
        this.timer = timer;
        this.timer.set();
        this.urls = urls;
        this.query = query;
        this.alive = true;
        this.index = 0;
    } 
    
    /** 
     * Used to try URL access before starting with sampling loop.
     * This method must be called to ensure the validity of the requested
     * URLs. The method accepts a vector of strings and returns a vector
     * of valid and accessible URL objects.
     */
    public static Vector tryURLs(Vector urls, String query, JComponent caller) {
        Vector u = new Vector(urls.size());
        
        Enumeration e = urls.elements();
        while (e.hasMoreElements()) {
            try {
                URL url = new URL((String) e.nextElement());
                URLConnection connection = url.openConnection();
                connection.setUseCaches(false);
            
                if (query != null) {
                    connection.setDoOutput(true);
                    PrintWriter output = new PrintWriter(connection.getOutputStream());
                    output.println(query);
                    output.close();
                }
            
                connection.getInputStream().close();
                u.addElement(url);
             } catch (Exception ex) {
                error(ex, caller);
             }
        }
        
        return u;
    }

    /** 
     * Sampling loop and thread execution.
     */
    public void run() {
        while (alive) {
            synchronized (this) {
                try {
                    this.setEnabled(false);
                    this.repaint();
                    wait(timer.delay());
                    this.setEnabled(true);
                    this.repaint();
                } catch (InterruptedException ignored) {}
            }
            
            try {
                URL url = getURL();
                Thread.currentThread().setName(url.toString());
                long sample = this.sample(url);
                if (alive) visualizer.add(sample);
            } catch (IOException e) {
                error(e, this);
                this.alive = false;
            }
        }
        
        this.setEnabled(false);
        this.repaint();
    }
    
    /** 
     * Causes this thread to die after finishing its current sample.
     */
    public void die() {
        this.alive = false;
    }

    /**
     * Returns the URL to test.
     */
    private URL getURL() {
        index = (++index < urls.size()) ? index : 0;
        return (URL) urls.elementAt(index);
    }
    
    /**
     * URL Sampling routine.
     */
    private long sample(URL url) throws IOException {
        URLConnection connection = url.openConnection();
        connection.setUseCaches(false);
        
        if (cookieManager != null) {
            String cookieHeader = cookieManager.getCookieHeaderForURL(url);
            if (cookieHeader != null) {
                connection.setRequestProperty("Cookie", cookieHeader);
            }
        }
        
        if (query != null) connection.setDoOutput(true);
        
        // Get current time
        long time =  System.currentTimeMillis();

        if (query != null) {
            PrintWriter output = new PrintWriter(connection.getOutputStream());
            output.println(query);
            output.close();
        }
        
        if (cookieManager != null) {
            // check for Set-Cookie headers from server (index 0 is the
            // HTTP repsonse code), which does not use "header: value" format
            for (int i = 1;; i++) {
                try {
                    if (connection.getHeaderFieldKey(i).equals("Set-Cookie")) {
                        cookieManager.addCookieFromHeader(connection.getHeaderField(i), url);
                    }
                } catch (NullPointerException e) {
                    break;
                }
            }
        }

        BufferedInputStream input = new BufferedInputStream(connection.getInputStream());
        for (int c = 0; c != -1; c = input.read());
        input.close();

        // Get elapsed time
        return System.currentTimeMillis() - time;
    }
    
    /**
     * Method used to signal an Exception to the user.
     */
    public static void error(Exception e, JComponent thrower) {
        JOptionPane.showMessageDialog(thrower, e, "Error", JOptionPane.ERROR_MESSAGE);
    }
}
